@@ -10,8 +10,9 @@ test_that("s2_lnglat objects can be created from and converted back to R objects
     as_s2_lnglat(s2_point(1, 0, 0)),
     s2_lnglat(0, 0)
   )
+
   expect_identical(
-    as_s2_lnglat(s2_point(NA, NA, NA)),
+    as_s2_lnglat(s2_point(NaN, NaN, NaN)),
     s2_lnglat(NaN, NaN)
   )
 })
